/*
  DAC8564.cpp - Library for controlling TI's DAC8569 8-channel 16-bit DAC.
  Created by Robert M. Learney, October 8, 2013.
*/

#include "DAC8564.h"
#include "boardfile.h"

DAC8564::DAC8564(void)
{

}

boolean DAC8564::init(int8_t pin, int8_t address)
{
  pinMode(pin, OUTPUT);
  digitalWrite(pin, HIGH);  //Start with CS held high
  _PIN = pin;  
  _ADDRESS = address;
  
  DAC8564::reference(_EXTERNAL); //Default to this reference mode - can change in main loop  
}

void DAC8564::reference(int32_t mode)
{
  SPI.beginTransaction(SPISettings(CHIP_MHZ, MSBFIRST, SPI_MODE1)); 
  
  digitalWrite(_PIN,LOW);
  SPI.transfer((mode >> 16) & 0x000000FF);
  SPI.transfer((mode >> 8) & 0x000000FF);
  SPI.transfer((mode >> 0) & 0x000000FF);
  digitalWrite(_PIN,HIGH);
}

void DAC8564::powerDown(int8_t channel) //Power down an individual channel
{
  SPI.beginTransaction(SPISettings(CHIP_MHZ, MSBFIRST, SPI_MODE1));   
   
  digitalWrite(_PIN,LOW);
   
  if(channel == DAC_ALL){
    SPI.transfer(0x35);
  }
  else{
    SPI.transfer(0x11 | channel | (_ADDRESS << 6));
  }

  SPI.transfer(0xC0); //Always power down in High-Z mode (change this if want to select mode)
  SPI.transfer(0x00); //Common LSBs to both options
  
  digitalWrite(_PIN,HIGH);
}

void DAC8564::clear(void)
{
  DAC8564::write(DAC_ALL, 0x0000);
}

void DAC8564::write(int8_t channel, uint16_t data) // Write to and update selected channel
{ 
  SPI.beginTransaction(SPISettings(CHIP_MHZ, MSBFIRST, SPI_MODE1));  

  uint16_t code = data;
  //uint16_t code = (uint16_t)(data*Vref); //Convert value in mV into output code
    
  digitalWrite(_PIN,LOW);
  
  if(channel == DAC_ALL){
    SPI.transfer( (_ADDRESS << 6) | 0x34 );
  }
  else{
    SPI.transfer( (_ADDRESS << 6) | 0x10 | channel);
  }
  
  SPI.transfer((code >> 8) & 0x00FF);
  SPI.transfer(code & 0x00FF);
  
  digitalWrite(_PIN,HIGH);
}
