/*
  DAC8564.h - Library for controlling TI's DAC8569 8-channel 16-bit DAC.
  Created by Robert M. Learney, October 8, 2013.
*/

#include <SPI.h>
#include "boardfile.h"

#ifndef _DAC8564_H
#define _DAC8564_H

//DAC Channel Selection
#define  DAC_A        0x00
#define  DAC_B        0x02
#define  DAC_C        0x04
#define  DAC_D        0x06
#define  DAC_ALL      0xFF

//DAC Power Down Modes
#define  _1K          0x40        //1k resistor to GND
#define  _100K        0x80        //100k resistor to GND
#define  _Z           0xC0        //High Z to ground

//DAC Reference Modes
#define  _ALWAYS      0x00011000  //Internal reference ALWAYS on
#define  _DEFAULT     0x00010000  //Internal reference powers down when DACs power down
#define  _EXTERNAL    0x00012000  //Use EXTERNAL reference (internal reference OFF)     

//Class definition
class DAC8564
{
  public:
    DAC8564(void);
    boolean init(int8_t pin, int8_t address);
    void reference(int32_t mode);
    void powerDown(int8_t channel); //To power down individual channels in different modes
    void clear(void);                               //Clear all channels to zero scale
    void write(int8_t channel, uint16_t data);   //Write data to and update a particular channel
  private:
    int8_t _PIN;
    int8_t _ADDRESS;
    const float Vref = 15.99975; //65535 divided by 4096mV reference
    //const float Vref = 13.107; //65535 divided by 5000mV reference
};

#endif
