/*
 LMP90080.h - Header file for controlling TI's LMP90080 multi-channel AFE
 
 Created by Robert M. Learney, September 1, 2015.

 */

#ifndef  _LMP90080_H
#define  _LMP90080_H

#include <SPI.h>
#include "boardfile.h"
#include "LUT_3850.h"

#define SPI_HANDSHAKECN   0x01
#define SPI_STREAMCN      0x03
#define SPI_DATA_ONLY_1   0x09
#define SPI_DATA_ONLY_2   0x0A
#define BGCALCN           0x10
#define SPI_DRDYBCN       0x11
#define ADC_AUXCN         0x12
#define SCALCN            0x17
#define LMP_DOUT          0x1A
#define CH_SCAN           0x1F

#define CH0       0x0
#define CH1       0x1
#define CH2       0x2
#define CH3       0x3
#define CH4       0x4
#define CH5       0x5
#define CH6       0x6

#define offsetCH0 0x30
#define offsetCH1 0x38
#define offsetCH2 0x40
#define offsetCH3 0x48

#define gainCH0   0x33
#define gainCH1   0x3B
#define gainCH2   0x43
#define gainCH3   0x4B

#define SCANMODE0 0b00000000
#define SCANMODE1 0b01000000
#define SCANMODE2 0b10000000
#define SCANMODE3 0b11000000

#define BURNOUT_ON  0b00000000
#define BURNOUT_OFF 0b10000000

#define VREF1 0b00000000
#define VREF2 0b01000000

#define _1SPS    0b00000000
#define _3SPS    0b00010000
#define _6SPS    0b00100000
#define _13SPS   0b00110000
#define _26SPS   0b01000000
#define _53SPS   0b01010000
#define _107SPS  0b01100000
#define _214SPS  0b01110000

#define FGA1    0b00000000
#define FGA2    0b00000010
#define FGA4    0b00000100
#define FGA8    0b00000110
#define FGA16   0b00001000
#define FGA32   0b00001010
#define FGA64   0b00001100
#define FGA128  0b00001110

#define BUFFER_ON   0b00000000
#define BUFFER_OFF  0b00000001

static float xvCelsius[3], yvCelsius[3];
static float xvBP[6], yvBP[6];

class LMP90080
{
  public:
    LMP90080(void);
    void init(int8_t cs);
    void offsetCorrection(void);
    void reset(void);
    void resetCalibrations(void);
    void BPgainCorrection(void);
    void powerOn(void);
    void powerOff(void);
    void channelInput(uint8_t channel, uint8_t data);
    void channelConfig(uint8_t channel, uint8_t data);
    uint16_t read(uint8_t address);
    int8_t readByte(uint8_t address);
    boolean write(uint8_t address, uint8_t data0);
    boolean write16(uint8_t address, uint16_t data);
   // boolean write(uint8_t address, uint8_t data0, uint8_t data1, uint8_t data2);
    int8_t getChan(void);
    uint16_t calculateTemp(int16_t RTDcode);
    uint16_t filterBessel_100Hz_5(int16_t BPcode);
    uint16_t filterButter_30Hz_3(int16_t BPcode);
    uint16_t getTemp(uint16_t value);
  
  private:
    int8_t _CS;
    uint16_t _RTDoffset;
    uint16_t _RTDgain;
    uint16_t _BPoffset;
    uint16_t _BPgain;
    boolean writeURA(uint8_t address);
    boolean writeLRA(uint8_t address, uint8_t data);
    boolean writeLRA(uint8_t address, uint8_t data0, uint8_t data1);
    boolean writeLRA(uint8_t address, uint8_t data0, uint8_t data1, uint8_t data2);
    boolean readURA(uint8_t address);
    uint8_t readLRA(uint8_t address);
};


#endif
