// MAX17043/44 library for Arduino
//
// Luca Dentella (http://www.lucadentella.it)

#include "MAX17043.h"
#include "Wire.h"
#include "boardfile.h"

int16_t MAX17043::getVCell() {

	byte MSB = 0;
	byte LSB = 0;
	
	readRegister(VCELL_REGISTER, MSB, LSB); //12-bit value from 0-5V in steps of 1.25mV
	int16_t value = (MSB << 4) | (LSB >> 4);
	return value;
 
	/* Commented out by RM Learney. return statement also moved
   * return map(value, 0x000, 0xFFF, 0, 50000) / 10000.0;
	 * //return value * 0.00125;
   */
}

byte MAX17043::getSoC() {
  
  byte MSB = 0;
  byte LSB = 0;

  readRegister(SOC_REGISTER, MSB, LSB); //16-bit value
  return MSB; // Return just the whole % values, stored in the MSB
}

/*
int16_t MAX17043::getSoC() {
	
	byte MSB = 0;
	byte LSB = 0;
	
	readRegister(SOC_REGISTER, MSB, LSB); //16-bit value
  int value = (MSB << 4) | (LSB >> 4);
  return value;

	 /* Commented out by RM Learney. Changed logic to create 'value' int16_t
	 * float decimal = LSB / 256.0;
   * return MSB + decimal;
}
*/

int16_t MAX17043::getVersion() {

	byte MSB = 0;
	byte LSB = 0;
	
	readRegister(VERSION_REGISTER, MSB, LSB);
	return (MSB << 8) | LSB;
}

byte MAX17043::getCompensateValue() {

	byte MSB = 0;
	byte LSB = 0;
	
	readConfigRegister(MSB, LSB);
	return MSB;
}

byte MAX17043::getAlertThreshold() {

	byte MSB = 0;
	byte LSB = 0;
	
	readConfigRegister(MSB, LSB);	
	return 32 - (LSB & 0x1F);
}

void MAX17043::setAlertThreshold(byte threshold) {

	byte MSB = 0;
	byte LSB = 0;
	
	readConfigRegister(MSB, LSB);	
	if(threshold > 32) threshold = 32;
	threshold = 32 - threshold;
	
	writeRegister(CONFIG_REGISTER, MSB, (LSB & 0xE0) | threshold);
}

boolean MAX17043::inAlert() {

	byte MSB = 0;
	byte LSB = 0;
	
	readConfigRegister(MSB, LSB);	
	return LSB & 0x20;
}

void MAX17043::clearAlert() {
/*
 * Edited to add the final 'writeRegister' command by RM Learney 08 Feb 2016
 */
	byte MSB = 0;
	byte LSB = 0;
	
	readConfigRegister(MSB, LSB);
  writeRegister(CONFIG_REGISTER, MSB, (LSB &= 0b11011111)); // Clear the ALERT bit in the LSB
}

void MAX17043::reset() {
	
	writeRegister(COMMAND_REGISTER, 0x00, 0x54);
}

void MAX17043::quickStart() {
	
	writeRegister(MODE_REGISTER, 0x40, 0x00);
}


void MAX17043::readConfigRegister(byte &MSB, byte &LSB) {

	readRegister(CONFIG_REGISTER, MSB, LSB);
}

void MAX17043::readRegister(byte startAddress, byte &MSB, byte &LSB) {

	Wire.beginTransmission(MAX17043_ADDRESS);
	Wire.write(startAddress);
	Wire.endTransmission();
	
	Wire.requestFrom(MAX17043_ADDRESS, 2);
	MSB = Wire.read();
	LSB = Wire.read();
}

void MAX17043::writeRegister(byte address, byte MSB, byte LSB) {

	Wire.beginTransmission(MAX17043_ADDRESS);
	Wire.write(address);
	Wire.write(MSB);
	Wire.write(LSB);
	Wire.endTransmission();
}
