/* 
 *  Library to sleep the ATMEGA328P
 *  
 *  RM Learney Nov 2015
 */

#include "sleepFunctions.h"
#include "boardfile.h"

void sleepFunctions::init(void) {

    ACSR = (1<<ACD); //Disable the analog comparator
    power_timer1_disable();
    power_timer2_disable(); //Needed for asynchronous 32kHz operation
    DIDR1 = (1<<AIN1D)|(1<<AIN0D); //Disable digital input buffer on AIN1/0 - ?remove this
    
    #ifndef DEBUG
      power_adc_disable();
      power_usart0_disable(); //Needed for serial.print
    #endif
}

void sleepFunctions::lightSleep(void) {   
  
  wdt_disable();  // Disable the Watchdog Timer

  power_timer0_disable(); //Needed for delay and millis() - if enabled will wake every 1msec 
  power_twi_disable(); // Disable I2C
  power_spi_disable(); // Disable SPI

  set_sleep_mode(SLEEP_MODE_STANDBY); // prepare for powerdown 
    
  noInterrupts(); // Following sequence is timing critical
    
  sleep_enable(); 

  //Need to change BOD fuse so don't need to turn off in software - saves 65usec each time
  MCUCR = bit (BODS) | bit (BODSE);    // Turn off brown-out enable in software
  MCUCR = bit (BODS);   //NB brown-out detection is automatically restarted at wakeup 

  interrupts(); // Re-enable inputs before sleeping
    
  sleep_cpu();   // Power down!
}

void sleepFunctions::deepSleep(void) { //If battery dying, go into lowest power mode
  
  wdt_disable();
  
  power_all_disable();

  DDRD &= B00000011;      // set Arduino pins 2-7 as inputs, leaves 0 & 1 (RX & TX) as is
  DDRB = 0x00;       // set pins 8 to 13 as inputs
  DDRC = 0x00;       // set analog pins as inputs
  PORTD |= 0x00;     // disable pullups on pins 1 to 7
  PORTB |= 0x00;     // disable pullups on pins 8 to 13
  PORTC |= 0x00;     // disable pullups on analog pins
    
  set_sleep_mode (SLEEP_MODE_PWR_DOWN);

  noInterrupts(); // Following sequence is timing critical
    
  sleep_enable(); 
    
  MCUCR = bit (BODS) | bit (BODSE);    // Turn off brown-out enable in software
  MCUCR = bit (BODS);   //NB brown-out detection is automatically restarted at wakeup 

  // Don't re-enable inputs before deep sleeping
    
  sleep_cpu();   // Power down!
}

void sleepFunctions::wakeupALL(void) {
  sleep_disable();
  power_all_enable();
}

void sleepFunctions::wakeupTimer0(void) {  
  sleep_disable();
  power_timer0_enable();
}

void sleepFunctions::wakeupI2C(void) {
  sleep_disable();
  power_twi_enable();
}

void sleepFunctions::wakeupSPI(void) {
  sleep_disable();
  power_spi_enable();
}

